<?php

namespace App\Controllers\Application;

use App\Controllers\BaseController;
use App\Libraries\ApiResponse;
use App\Libraries\Auth;
use App\Libraries\ExcecaoCertificaSe;
use App\Libraries\GerarCertificado;
use App\Models\CertificadosInscritosModel;
use App\Models\CertificadosModel;
use App\Models\CertificadosModelosModel;
use App\Models\InscritosModel;
use Exception;
use ReflectionException;

class Application extends BaseController
{
    public function index(): string
    {
        if ($this->request->isAJAX()) {
            $reposta = ApiResponse::init();

            $page = $this->request->getPost('page', FILTER_SANITIZE_NUMBER_INT) ?? 1;

            try {
                $model = new CertificadosInscritosModel();
                $certificados = $model
                    ->where('inscritos_id', Auth::get('id'))
                    ->orderBy('id', 'desc')
                    ->paginate(10, 'default', $page);

                $data = array_map(function ($certificado) {
                    return array_merge((array)$certificado, [
                        'Certificado' => (new CertificadosModel())->find($certificado->certificados_id),
                        'CertificadoModelo' => (new CertificadosModelosModel())->find($certificado->certificados_id),
                    ]);
                }, $certificados);

                $reposta->success(self::$messageSuccess, $data, null, null, $model->pager->getDetails());
            } catch (Exception $e) {
                $reposta->error(self::$messageError);
            }
        }
        return $this->render();
    }

    public function minhaConta()
    {
        if ($this->request->isAJAX()) {
            $model = new InscritosModel();
            $inscrito = $model->find(Auth::get('id'));
            $resposta = ApiResponse::init();

            $rules = [
                'nome' => 'required|max_length[255]',
                'email' => 'required|max_length[254]|valid_email',
                'senha' => 'permit_empty|min_length[8]',
            ];

            if (!$this->validate($rules)) {
                $resposta->error(self::$messageError, $this->validator->getErrors());
            }

            try {
                $data = $this->validator->getValidated();
                foreach ($data as $key => $value) {
                    if (!empty($value)) {
                        $inscrito->$key = $value;
                    }
                }

                $model->save($inscrito);

                $resposta->success(self::$messageSuccess, null, current_url());
            } catch (ReflectionException $e) {
                $resposta->error(self::$messageError, [$e->getMessage()]);
            }
        }
        return $this->render();
    }

    public function certificado()
    {
        try {
            $code = $this->request->getUri()->getSegment(3);

            $model = new CertificadosInscritosModel();
            $certificado = $model->where('codigo_validacao', $code)->first();

            if (!$certificado) {
                ExcecaoCertificaSe::lancar('Certificado não encontrado');
            }

            $certificado->baixado = 1;
            $certificado->quantidade_downloads++;
            $model->save($certificado);
            $gerador = GerarCertificado::iniciar($certificado->clientes_id, $certificado->certificados_id, $certificado->id);
            $gerador->gerar('D');
        } catch (ExcecaoCertificaSe|ReflectionException $e) {
            return redirect()->route('app.to', ['index?error=' . urlencode($e->getMessage())]);
        }
    }
}