<?php

namespace App\Controllers\Dashboard;

use App\Controllers\BaseController;
use App\Enums\MetodosPagamentoEnum;
use App\Enums\StatusEnum;
use App\Libraries\Auth;
use App\Libraries\Data;
use App\Libraries\ExcecaoCertificaSe;
use App\Models\ClientesCreditoHistoricoModel;
use App\Models\ClientesPerfilModel;
use App\Models\CobrancasModel;
use App\Models\MetodosPagamentoModel;
use App\Models\ProdutosModel;
use App\ThirdParty\Cobrancas\Cobranca;
use App\ThirdParty\Cobrancas\DTO\CobrancaDTO;
use stdClass;

class Creditos extends BaseController
{
    public function index()
    {
        return $this->render();
    }

    public function comprar()
    {
        $modelProdutos = new ProdutosModel();
        $modelMetodosPagemento = new MetodosPagamentoModel();
        $produtoId = $this->request->getUri()->getSegment(4);

        try {
            $pagePagamento = $this->request->getUri()->getSegment(5);
        } catch (\Exception $exception) {
            $pagePagamento = null;
        }

        $includePage = null;
        $perfil = null;

        if ($this->perfilCompleto($includePage, $perfil)) {
            if ($produtoId) {
                $produto = $modelProdutos->where([
                    'id' => $produtoId,
                    'status' => StatusEnum::ATIVO->value
                ])->first();

                if (!$produto) {
                    return redirect()->route('dash.to', ['creditos/comprar?error=' . urlencode('Produto não encontrado!')]);
                }

                $metodosPagemento = $modelMetodosPagemento->where('status', StatusEnum::ATIVO->value)->findAll();

                $includePage = 'dashboard/creditos/comprar/metodo-pagamento';
            } else {
                $where = ['status' => StatusEnum::ATIVO->value];
                $qtd = $this->request->getGet('quantidade');
                if ($qtd) {
                    $where['quantidade >='] = $qtd;
                }
                $produtos = $modelProdutos->where($where)->findAll();
                $includePage = 'dashboard/creditos/comprar/produtos';
            }
        }

        if ($pagePagamento && isset($produto)) {
            if (method_exists($this, $pagePagamento)) {
                $dataVencimento = Data::obj()->modify('+1 week')->format('Y-m-d');
                $cobranca = CobrancasModel::criar($pagePagamento, $produto, Auth::user(), $dataVencimento);
                return $this->$pagePagamento($cobranca, $produto, $modelMetodosPagemento, $dataVencimento);
            } else {
                die('Método não existe');
            }
        }

        return $this->render([
            'produto' => $produto ?? null,
            'produtos' => $produtos ?? [],
            'metodosPagemento' => $metodosPagemento ?? [],
            'includePage' => $includePage,
            'fromCompra' => true,
            'perfil' => $perfil
        ]);
    }

    public function boleto(object $cobranca, stdClass $produto, MetodosPagamentoModel $model, ?string $dataVencimento = null)
    {
        $where = [
            'status' => StatusEnum::ATIVO->value,
            'componente' => MetodosPagamentoEnum::BOLETO->value
        ];

        $metodoPagamento = $model->where($where)->first();

        if (!$metodoPagamento) {
            die('Método de pagamento não encontrado');
        }

        try {
            $cobrancaDTO = new CobrancaDTO($cobranca);
            $cobrancaDTO->setHomologacao(ENVIRONMENT === 'development');
            $cobrancaDTO->setUrlCallback($metodoPagamento);
            $cobrancaDTO->addProduto($produto);

            $cobranca = Cobranca::init($metodoPagamento->integrador, $cobrancaDTO);
            $cobranca->criarCobrancaBoleto($dataVencimento);
            return $cobranca->getBoleto();
        } catch (ExcecaoCertificaSe $e) {
            dd($e->getMessage());
        }
    }

    public function pix(object $cobranca, stdClass $produto, MetodosPagamentoModel $model, ?string $dataVencimento = null)
    {
        $where = [
            'status' => StatusEnum::ATIVO->value,
            'componente' => MetodosPagamentoEnum::BOLETO->value
        ];

        $metodoPagamento = $model->where($where)->first();

        if (!$metodoPagamento) {
            die('Método de pagamento não encontrado');
        }

        try {
            $cobrancaDTO = new CobrancaDTO($cobranca);
            $cobrancaDTO->setHomologacao(ENVIRONMENT === 'development');
            $cobrancaDTO->setUrlCallback($metodoPagamento);
            $cobrancaDTO->addProduto($produto);
            $cobranca = Cobranca::init($metodoPagamento->integrador, $cobrancaDTO);
            $cobranca->criarCobrancaPix($dataVencimento);
            return $cobranca->getQrCode();
        } catch (ExcecaoCertificaSe $e) {
            dd($e->getMessage());
        }
    }

    private function perfilCompleto(&$includePage, &$perfil): bool
    {
        $includePage = 'dashboard/minha-conta/form-perfil';
        $perfil = Auth::perfil();
        return !empty($perfil->documento_numero) && !empty($perfil->contatos) && !empty($perfil->enderecos);
    }

    public function extrato()
    {
        return $this->render();
    }
}