<?php

namespace App\Controllers\Dashboard;

use App\Controllers\BaseController;
use App\Libraries\Suporte;
use App\Models\InscritosModel;
use Config\Database;

class Inscritos extends BaseController
{
    public function index()
    {
        return $this->render();
    }

    public function certificados()
    {
        $inscritos_id = $this->request->getUri()->getSegment(4);
        if (($path = $this->request->getUri()->getSegment(5)) !== '') {
            if ($path == "datatable") {
                $this->datatableCertificadosInscritos();
                die;
            }
        }

        $inscrito = (new InscritosModel())->find($inscritos_id);

        if (!$inscrito) {
            return redirect()->route('dash.to', ['inscritos?error=' . urlencode(self::$messageDenyAccess)]);
        }

        return $this->render([
            'inscritos_id' => $inscrito->id,
            'inscritos' => $inscrito,
        ]);
    }

    public function datatableCertificadosInscritos()
    {
        $db = Database::connect();
        $table = $this->request->getVar('origem');
        $tableJoin = $this->request->getVar('origem_join');

        $builder = $db->table($table);

        $start = $this->request->getVar('start');
        $length = $this->request->getVar('length');
        $columns = $this->request->getVar('columns');
        $search = $this->request->getVar('search');
        $order = $this->request->getVar('order');
        $clientes_id = $this->request->getVar('clientes_id');
        $inscritos_id = $this->request->getVar('inscritos_id');

        $dataColumns = [];
        $searchable = null;
        $orderable = null;

        foreach ($columns as $index => $column) {
            if (!$column['data']) {
                continue;
            }

            $arr = explode(".", $column['name']);

            if (count($arr) == 2) {
                $col = $column['name'];
                $dataColumns[] = "{$col} AS " . $column['data'];
            } else {
                $col = "{$table}.{$column['data']}";
                $dataColumns[] = "{$col} AS " . $column['name'];
            }


            if ($column['searchable'] && !empty($search['value'])) {
                $searchable[$col] = $search['value'];
            }
            if ($column['orderable']) {
                $orderable[$index] = $col;
            }
        }

        $builder->distinct()->select($dataColumns);

        if (!empty($searchable)) {
            $builder->orLike($searchable);
        }

        if (!empty($order)) {
            foreach ($order as $ord) {
                if (isset($orderable[$ord['column']])) {
                    $builder->orderBy($orderable[$ord['column']], $ord['dir']);
                }
            }
        }

        $cond = "{$table}.id = {$tableJoin}.certificados_id AND {$table}.clientes_id = {$clientes_id}";
        $builder->join($tableJoin, $cond);

        $where = "{$table}.deleted_at IS NULL";

        if ($inscritos_id) {
            $builder->where("{$tableJoin}.inscritos_id", $inscritos_id);
            $where = "({$tableJoin}.inscritos_id = {$inscritos_id} AND {$tableJoin}.deleted_at IS NULL)";
        }

        $data = $builder->where($where)->get($length, $start);
        $recordsTotal = $builder->join($tableJoin, $cond)->where($where)->countAllResults();
        $recordsFiltered = $builder->join($tableJoin, $cond)->where($where)->countAllResults();

        $datatable = [
            'draw' => (int)$this->request->getVar('draw'),
            'recordsTotal' => (int)$recordsTotal,
            'recordsFiltered' => (int)$recordsFiltered,
            'data' => array_map(function ($item) {
                return array_merge(['DT_RowId' => $item['id']], $item);
            }, $data->getResult('array')),
        ];

        Suporte::json($datatable);
    }

    public function datatable()
    {
        $db = Database::connect();

        $table = $this->request->getVar('origem');
        $tableJoin = $this->request->getVar('origem_join');

        $builder = $db->table($table);

        $start = $this->request->getVar('start');
        $length = $this->request->getVar('length');
        $columns = $this->request->getVar('columns');
        $search = $this->request->getVar('search');
        $order = $this->request->getVar('order');
        $clientes_id = $this->request->getVar('clientes_id');

        $dataColumns = [];
        $searchable = null;
        $orderable = null;

        foreach ($columns as $index => $column) {
            if (!$column['data']) {
                continue;
            }

            $dataColumns[] = "{$table}.{$column['data']} AS " . ($column['name'] ?? $column['data']);

            if ($column['searchable'] && !empty($search['value'])) {
                $searchable["{$table}.{$column['data']}"] = $search['value'];
            }
            if ($column['orderable']) {
                $orderable[$index] = "{$table}.{$column['data']}";
            }
        }

        $builder->distinct()->select($dataColumns);

        if (!empty($searchable)) {
            $builder->orLike($searchable);
        }

        if (!empty($order)) {
            foreach ($order as $ord) {
                if (isset($orderable[$ord['column']])) {
                    $builder->orderBy($orderable[$ord['column']], $ord['dir']);
                }
            }
        }

        $builder->join($tableJoin, "{$table}.id = {$tableJoin}.inscritos_id");

        $where = "{$table}.deleted_at IS NULL";

        if ($clientes_id) {
            $builder->where("{$tableJoin}.clientes_id", $clientes_id);
            $where = "({$tableJoin}.clientes_id = {$clientes_id} AND {$tableJoin}.deleted_at IS NULL)";
        }

        $data = $builder->where($where)->get($length, $start);

        $recordsTotal = $builder->join($tableJoin, "{$table}.id = {$tableJoin}.inscritos_id")->where($where)->countAllResults();
        $recordsFiltered = $builder->join($tableJoin, "{$table}.id = {$tableJoin}.inscritos_id")->where($where)->countAllResults();

        $datatable = [
            'draw' => (int)$this->request->getVar('draw'),
            'recordsTotal' => (int)$recordsTotal,
            'recordsFiltered' => (int)$recordsFiltered,
            'data' => array_map(function ($item) {
                return array_merge(['DT_RowId' => $item['id']], $item);
            }, $data->getResult('array')),
        ];

        Suporte::json($datatable);
    }
}