<?php

namespace App\Libraries;

class ApiResponse
{
    private array $resposta = [];

    public function success(
        string            $mensagem = '',
        array|object|null $data = null,
        ?string           $redirect = null,
        ?string           $token = null,
        mixed             $details = null
    ): void
    {
        if (is_object($data)) {
            $data = (array)$data;
        }
        $this->setResposta($redirect, true, $mensagem, empty($data) ? [] : $data, null, $token, $details);
        $this->render();
    }

    public function error(
        string  $mensagem = '',
        ?array  $erros = null,
        ?string $redirect = null
    ): void
    {
        $this->setResposta($redirect, false, $mensagem, null, empty($erros) ? [] : array_values($erros));
        $this->render();
    }

    public function render(): void
    {
        Suporte::json($this->getResposta(), 200);
    }

    public static function init(): self
    {
        return new self();
    }

    private function getResposta(): array
    {
        if (empty($this->resposta)) {
            $this->error(
                'Houve um erro. Por favor, tente novamente. Se o problema persistir, entre em contato com o administrador do sistema.'
            );
        }
        return $this->resposta;
    }

    private function setResposta(
        ?string $redirect = null,
        bool    $success = true,
        string  $message = '',
        ?array  $data = null,
        ?array  $erros = null,
        ?string $token = null,
        mixed   $details = null
    ): void
    {
        helper('nortedevbr');
        $this->resposta = array_filter([
            'success' => $success,
            'message' => t($message),
            'data' => $data,
            'errors' => $erros,
            'redirect' => $redirect,
            'token' => $token,
            'meta' => $details
        ], function ($value) {
            return $value !== '' && $value !== null;
        });
    }
}