<?php

namespace App\Models;

use App\Enums\StatusCertificadosEnum;
use CodeIgniter\Model;

class CertificadosInscritosModel extends Model
{
    protected $table = 'certificados_inscritos';
    protected $primaryKey = 'id';
    protected $useAutoIncrement = true;
    protected $returnType = 'object';
    protected $useSoftDeletes = true;
    protected $protectFields = true;
    protected $allowedFields = [
        'id',
        'clientes_id',
        'certificados_id',
        'inscritos_id',
        'gerado',
        'baixado',
        'quantidade_downloads',
        'quantidade_acessos',
        'codigo_validacao',
        'nome',
        'email',
        'senha',
        'texto',
        'created_at',
        'updated_at',
    ];

    protected bool $allowEmptyInserts = false;
    protected bool $updateOnlyChanged = true;

    protected array $casts = [];
    protected array $castHandlers = [];

    // Dates
    protected $useTimestamps = false;
    protected $dateFormat = 'datetime';
    protected $createdField = 'created_at';
    protected $updatedField = 'updated_at';
    protected $deletedField = 'deleted_at';

    // Validation
    protected $validationRules = [];
    protected $validationMessages = [];
    protected $skipValidation = false;
    protected $cleanValidationRules = true;

    // Callbacks
    protected $allowCallbacks = true;
    protected $beforeInsert = ['beforeInsert'];
    protected $afterInsert = ['insertInscrito'];
    protected $beforeUpdate = ['beforeUpdate'];
    protected $afterUpdate = [];
    protected $beforeFind = [];
    protected $afterFind = [];
    protected $beforeDelete = [];
    protected $afterDelete = [];

    public function beforeInsert($data): array
    {
        $data['data']['gerado'] = StatusCertificadosEnum::AGUARDANDO->value;
        $data['data']['baixado'] = 0;
        $data['data']['quantidade_downloads'] = 0;
        $data['data']['quantidade_acessos'] = 0;
        return $data;
    }

    public function beforeUpdate($data): array
    {
        if (isset($data['data']['gerado'])) {
            $cancelado = StatusCertificadosEnum::isCancelado((int)$data['data']['gerado']);
        } else {
            $cancelado = false;
        }

        if (!empty($data['data']['codigo_validacao']) && !$cancelado) {
            $data['data']['gerado'] = match ((int)$data['data']['gerado']) {
                StatusCertificadosEnum::ATUALIZADO_AGUARDANDO->value, StatusCertificadosEnum::ATUALIZADO_GERADO->value => StatusCertificadosEnum::ATUALIZADO_GERADO->value,
                default => StatusCertificadosEnum::GERADO->value,
            };
        } elseif (!$cancelado && isset($data['data']['gerado'])) {
            $data['data']['gerado'] = StatusCertificadosEnum::ATUALIZADO_AGUARDANDO->value;
        } elseif (!$cancelado || !isset($data['data']['gerado'])) {
            $data['data']['gerado'] = StatusCertificadosEnum::AGUARDANDO->value;
        }

        return $data;
    }

    public function insertInscrito($data): array
    {
        $modelo = new InscritosModel();
        $entity = $modelo->where('email', $data['data']['email'])->first();
        if (!$entity) {
            $modelo->save([
                'email' => $data['data']['email'],
                'nome' => $data['data']['nome'],
                'senha' => $data['data']['senha'],
                'senha_hash' => password_hash($data['data']['senha'], PASSWORD_DEFAULT),
            ]);
            $inscritosId = $modelo->getInsertID();
        } else {
            $inscritosId = $entity->id;
        }

        $data['data']['inscritos_id'] = $inscritosId;
        $this->update($data['id'], $data['data']);

        return $data;
    }
}
