<?php

namespace App\Models;

use App\Enums\CreditosEnum;
use App\Libraries\ExcecaoCertificaSe;
use CodeIgniter\Model;
use ReflectionException;

class ClientesCreditoModel extends Model
{
    protected $table = 'clientes_credito';
    protected $primaryKey = 'id';
    protected $useAutoIncrement = true;
    protected $returnType = 'object';
    protected $useSoftDeletes = true;
    protected $protectFields = true;
    protected $allowedFields = [
        'id',
        'clientes_id',
        'saldo',
        'saldo_anterior',
        'created_at',
        'updated_at',
    ];

    protected bool $allowEmptyInserts = false;
    protected bool $updateOnlyChanged = true;

    protected array $casts = [];
    protected array $castHandlers = [];

    // Dates
    protected $useTimestamps = false;
    protected $dateFormat = 'datetime';
    protected $createdField = 'created_at';
    protected $updatedField = 'updated_at';
    protected $deletedField = 'deleted_at';

    // Validation
    protected $validationRules = [];
    protected $validationMessages = [];
    protected $skipValidation = false;
    protected $cleanValidationRules = true;

    // Callbacks
    protected $allowCallbacks = true;
    protected $beforeInsert = [];
    protected $afterInsert = [];
    protected $beforeUpdate = [];
    protected $afterUpdate = [];
    protected $beforeFind = [];
    protected $afterFind = [];
    protected $beforeDelete = [];
    protected $afterDelete = [];

    public static function adicionar(object $cobranca): static
    {
        $model = new self();
        $modelHistorico = new ClientesCreditoHistoricoModel();

        if ($modelHistorico->where('cobrancas_id', $cobranca->id)->countAllResults() > 0) {
            return $model;
        }

        $data = $model->where('clientes_id', $cobranca->clientes_id)->first();

        if ($data) {
            $data->saldo_anterior = $data->saldo;
            $data->saldo = $data->saldo + $cobranca->produto_quantidade;
        } else {
            $data = (object)[
                'clientes_id' => $cobranca->clientes_id,
                'saldo' => $cobranca->produto_quantidade,
                'saldo_anterior' => 0
            ];
        }

        if ($model->save($data)) {
            $modelHistorico->save([
                'clientes_id' => $cobranca->clientes_id,
                'cobrancas_id' => $cobranca->id,
                'saldo' => $data->saldo,
                'saldo_anterior' => $data->saldo_anterior,
                'acao' => CreditosEnum::ENTRADA->value,
            ]);
        }

        return $model;
    }

    /**
     * @throws ReflectionException
     * @throws ExcecaoCertificaSe
     */
    public function subtrair(string|int $clienteId, int $quantidade): int
    {
        $model = new self();
        $modelHistorico = new ClientesCreditoHistoricoModel();

        $data = $model->where('clientes_id', $clienteId)->first();

        if ($data) {
            if ($data->saldo > $quantidade) {
                $data->saldo_anterior = $data->saldo;
                $data->saldo = $data->saldo - $quantidade;
                if ($model->save($data)) {
                    $modelHistorico->save([
                        'clientes_id' => $clienteId,
                        'saldo' => $data->saldo,
                        'saldo_anterior' => $data->saldo_anterior,
                        'acao' => CreditosEnum::SAIDA->value,
                    ]);
                }
            } else {
                ExcecaoCertificaSe::lancar('Saldo insuficiente');
            }
        }

        return 0;
    }
}
