<?php

namespace App\Models;

use App\Enums\MetodosPagamentoEnum;
use App\Enums\StatusEnum;
use App\Libraries\Data;
use CodeIgniter\Model;
use ReflectionException;
use stdClass;

class CobrancasModel extends Model
{
    protected $table = 'cobrancas';
    protected $primaryKey = 'id';
    protected $useAutoIncrement = true;
    protected $returnType = 'object';
    protected $useSoftDeletes = true;
    protected $protectFields = true;
    protected $allowedFields = [
        'id',
        'clientes_id',
        'produtos_id',
        'metodos_pagamento_id',
        'metodo',
        'produto_nome',
        'produto_valor',
        'produto_quantidade',
        'pacelas',
        'valor_parcela',
        'status',
        'tipo_liquidado',
        'data_vencimento',
        'data_pagamento',
        'data_liquidado',
        'created_at',
        'updated_at',
    ];

    protected bool $allowEmptyInserts = false;
    protected bool $updateOnlyChanged = true;

    protected array $casts = [];
    protected array $castHandlers = [];

    // Dates
    protected $useTimestamps = false;
    protected $dateFormat = 'datetime';
    protected $createdField = 'created_at';
    protected $updatedField = 'updated_at';
    protected $deletedField = 'deleted_at';

    // Validation
    protected $validationRules = [];
    protected $validationMessages = [];
    protected $skipValidation = false;
    protected $cleanValidationRules = true;

    // Callbacks
    protected $allowCallbacks = true;
    protected $beforeInsert = [];
    protected $afterInsert = [];
    protected $beforeUpdate = ['beforeUpdate'];
    protected $afterUpdate = [];
    protected $beforeFind = [];
    protected $afterFind = [];
    protected $beforeDelete = [];
    protected $afterDelete = [];

    public function beforeUpdate($data): array
    {
        if ($data['data']['status'] === StatusEnum::PAGO->value) {
            ClientesCreditoModel::adicionar((object)$data['data']);
        }
        return $data;
    }

    public static function criar(
        string  $metodoPagamento,
        object  $produto,
        object  $user,
        ?string $dataVencimento = null,
        ?int    $parcelas = null,
        ?int    $valorParcelas = null
    ): object|null
    {
        try {
            $where = [
                'status' => StatusEnum::ATIVO->value,
                'componente' => $metodoPagamento
            ];

            $metodosPagamentoModel = new MetodosPagamentoModel();
            $metodoPagamento = $metodosPagamentoModel->where($where)->first();

            $parcelas = $parcelas ?? 1;

            $data = new stdClass();
            $data->clientes_id = $user->id;
            $data->produtos_id = $produto->id;
            $data->metodos_pagamento_id = $metodoPagamento->id;
            $data->metodo = $metodoPagamento->componente;
            $data->produto_nome = $produto->nome;
            $data->produto_valor = $produto->valor;
            $data->produto_quantidade = $produto->quantidade;
            $data->pacelas = $parcelas;
            $data->valor_parcela = $valorParcelas ?? (int)ceil($produto->valor / $parcelas);
            $data->status = StatusEnum::AGUARDANDO->value;
            $data->tipo_liquidado = null;
            $data->data_vencimento = $dataVencimento ?? Data::agora();
            $data->data_pagamento = null;
            $data->data_liquidado = null;

            $model = new CobrancasModel();
            $model->save($data);
            $id = $model->getInsertID();
            return $model->find($id);
        } catch (ReflectionException $e) {
            return null;
        }
    }
}
