<?php

namespace App\ThirdParty\Cobrancas\Integradores;

use DateTime;
use OpenBoleto\Agente;
use OpenBoleto\Banco\BancoDoBrasil;
use App\Libraries\Suporte;
use App\ThirdParty\Cobrancas\DTO\ConsultaPagamentoDTO;
use App\ThirdParty\Cobrancas\DTO\WebhookDTO;
use Mpdf\QrCode\QrCode;
use Mpdf\QrCode\Output;

class FAKE extends IntegradorAbstract implements IntegradorInterface
{
    /**
     * @return $this
     */
    public function criarCobrancaCartaoCredito(): static
    {
        return $this;
    }

    /**
     * @return $this
     */
    public function criarCobrancaBoleto(?string $dataVencimento = null): static
    {
        $sacado = new Agente('Fernando Maia', '023.434.234-34', 'ABC 302 Bloco N', '72000-000', 'Brasília', 'DF');
        $cedente = new Agente('Empresa de cosméticos LTDA', '02.123.123/0001-11', 'CLS 403 Lj 23', '71000-000', 'Brasília', 'DF');

        $boleto = new BancoDoBrasil([
            // Parâmetros obrigatórios
            'dataVencimento' => $dataVencimento ? new DateTime($dataVencimento) : (new Datetime)->modify('+7 day'),
            'valor' => Suporte::intToFloat($this->getCobranca()->getProdutos()[0]->valor),
            'sequencial' => 1234567, // Para gerar o nosso número
            'sacado' => $sacado,
            'cedente' => $cedente,
            'agencia' => 1724, // Até 4 dígitos
            'carteira' => 18,
            'conta' => 10403005, // Até 8 dígitos
            'convenio' => 1234, // 4, 6 ou 7 dígitos
            'qrCode' => 'otpauth://totp/test?secret=B3JX4VCVJDVNXNZ5&issuer=chillerlan.net', // Caso o Boleto tenha registro para uso de QrCode Pix
        ]);

        $this->setBoleto($boleto->getOutput());

        return $this;
    }

    /**
     * @return $this
     */
    public function criarCobrancaPix(?string $dataVencimento = null): static
    {
        $qrCode = new QrCode('Lorem ipsum sit dolor');
        $output = new Output\Html();
        $this->setQrCode($output->output($qrCode));

        return $this;
    }

    /**
     * @param int|string|null $transacaoId
     * @return $this
     */
    public function consultarCobranca(int|string|null $transacaoId = null): static
    {
        return $this;
    }

    /**
     * @param int|string|null $transacaoId
     * @return $this
     */
    public function cancelarCobranca(int|string|null $transacaoId = null): static
    {

        return $this;
    }

    /**
     * @param ConsultaPagamentoDTO $consultaPagamento
     * @return $this
     */
    public function consultarPagamentos(ConsultaPagamentoDTO $consultaPagamento): static
    {

        return $this;
    }

    /**
     * @param WebhookDTO $webhook
     * @return $this
     */
    public function webhook(WebhookDTO $webhook): static
    {

        return $this;
    }
}