document.addEventListener('DOMContentLoaded', () => {
    document.querySelector('.main-menu-mobile-btn').addEventListener('click', (e) => {
        let mainMenu = document.querySelector('.main-menu');
        mainMenu.classList.toggle('main-menu-ativo');
    });

    document.querySelector('.menu').querySelectorAll('a')?.forEach((a) => {
        if (a.href === window.location.href) {
            a.classList.add('ativo');
        }
    });

    document.getElementById('app').addEventListener('click', event => {
        if (event.target.classList.contains('delete') || event.target.closest('.delete')?.classList.contains('delete')) {
            event.preventDefault();
            let url_to = null;
            if (event.target.classList.contains('delete')) {
                url_to = event.target.href;
            } else {
                url_to = event.target.closest('.delete').href;
            }
            let url = new URL(url_to);
            url_to = url.pathname;

            swal.fire({
                title: "Tem certeza?",
                text: "Você não poderá reverter isso!",
                icon: "question",
                showCancelButton: true,
                confirmButtonText: "Sim, exclua-o!",
                cancelButtonText: "Não, cancele!",
                reverseButtons: true,
                showLoaderOnConfirm: true,
                preConfirm: async () => {
                    try {
                        const bearerToken = document.head.querySelector('meta[name="token"]');
                        const response = await fetch(url_to, {
                            headers: {
                                'X-Requested-With': 'XMLHttpRequest',
                                'Authorization': `Bearer ${bearerToken.getAttribute('content')}`
                            }
                        });
                        if (!response.ok) {
                            return swal.showValidationMessage(`${JSON.stringify(await response.json())}`);
                        }
                        return response.json();
                    } catch (error) {
                        swal.showValidationMessage(`Request failed: ${error}`);
                    }
                },
                allowOutsideClick: () => !swal.isLoading()
            }).then((result) => {
                if (result.isConfirmed && result.value.success) {
                    swal.fire({
                        text: result.value.message ?? "Excluído com sucesso.",
                        icon: "info",
                        toast: true,
                        showConfirmButton: false,
                        timer: 1500,
                        timerProgressBar: true,
                    }).then(() => {
                        if (result.value.redirect) {
                            swal.showLoading();
                            window.location.href = result.value.redirect;
                        }
                        $.fn.dataTable.tables().forEach(table => {
                            let tableInstance = $(table).DataTable();
                            tableInstance.ajax.reload(null, false);
                        });
                    });
                } else if (result.dismiss === Swal.DismissReason.cancel || !result.value.success) {
                    swal.fire({
                        text: result.value.message ?? "Cancelado!",
                        icon: result.value.message ? "error" : "success",
                        toast: true,
                        showConfirmButton: false,
                        timer: result.value.message ? 5000 : 1500,
                        timerProgressBar: true,
                    });
                }
            });
        }
    });
});


const alertPlaceholder = document.getElementById('liveAlertPlaceholder')

function alertNotify(message, type) {
    const wrapper = document.createElement('div');
    const alertId = (comprimento = 10) => {
        let caracteres = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789', senha = '';
        for (let i = 0; i < comprimento; i++) {
            let indiceAleatorio = Math.floor(Math.random() * caracteres.length);
            senha += caracteres.charAt(indiceAleatorio);
        }
        return senha;
    }
    const id = alertId();
    wrapper.innerHTML = [`<div id="${id}" class="alert alert-${type} alert-dismissible fade show" role="alert">`, `   <div>${message}</div>`, '   <button type="button" class="btn-close" data-bs-dismiss="alert" aria-label="Close"></button>', '</div>'].join('');
    alertPlaceholder.append(wrapper);
    const myAlert = document.getElementById(id)
    myAlert.addEventListener('closed.bs.alert', event => {
        let url = new URL(window.location.href);
        window.location.href = url.pathname;
    });
}